'use strict';

/**
 * 自动生成代码
 * 2019-5-23
 */

// #region [import]

const fs = require('fs');
const path = require('path');
const { Access, coder } = require('koa-rtapi');

// #endregion

// 构建配置
const map = [
    { file: '_coder/_apiAccess.js', single: true, target: 'api/xf-api/_apiAccess.js' },
    { file: '_coder/_dbDoc.md', single: true, target: 'www/xf-view/_dbDoc.md' },
    { file: '_coder/api.js', single: false, target: 'api/xf-api/{TABLE_NAME}.api.js' },

    { file: '_coder/home/home.css', single: true, target: 'www/xf-view/home/home.css' },
    { file: '_coder/home/home.html', single: true, target: 'www/xf-view/home/home.html' },
    { file: '_coder/home/home.js', single: true, target: 'www/xf-view/home/home.js' },

    { file: '_coder/grid/grid.css', single: false, target: 'www/xf-view/{TABLE_NAME}-grid/{TABLE_NAME}-grid.css' },
    { file: '_coder/grid/grid.html', single: false, target: 'www/xf-view/{TABLE_NAME}-grid/{TABLE_NAME}-grid.html' },
    { file: '_coder/grid/grid.js', single: false, target: 'www/xf-view/{TABLE_NAME}-grid/{TABLE_NAME}-grid.js' },

    { file: '_coder/edit/edit.css', single: false, target: 'www/xf-view/{TABLE_NAME}-edit/{TABLE_NAME}-edit.css' },
    { file: '_coder/edit/edit.html', single: false, target: 'www/xf-view/{TABLE_NAME}-edit/{TABLE_NAME}-edit.html' },
    { file: '_coder/edit/edit.js', single: false, target: 'www/xf-view/{TABLE_NAME}-edit/{TABLE_NAME}-edit.js' }
];

module.exports = class {

    // 构建配置
    constructor() {
        this.files = [];
        const arr = __dirname.replace(/\\/g, '/').split('/');
        arr.length -= 1;

        // 操作的根路径
        this.root = arr.join('/');
    }

    // 保存文件
    save(pth, code) {
        const t = this;
        const dir = path.dirname(pth);
        const arr = dir.split('/');
        const len = arr.length;

        let p = '';
        for (let i = 0; i < len; i++) {
            p = p + arr[i] + '/';
            if (!fs.existsSync(p)) {
                fs.mkdirSync(p);
            }
        }

        fs.writeFileSync(pth, code);
        t.files.push(pth);

    }

    // 获取数据库结构
    async getSchema(body) {
        const t = this;

        if (!body.host) {
            Object.assign(body, t.ctx.env.mysql['db1']);
        }

        // 获取tables
        let sql1 = "SELECT * FROM information_schema.TABLES WHERE TABLE_SCHEMA=database()";
        if (body.hd) {
            sql1 += ` AND TABLE_NAME like '${body.hd}%'`;
        }
        const tables = await new Access(body).setCtx(t.ctx).query(sql1);

        // 获取字段
        const sql2 = 'SELECT * FROM information_schema.COLUMNS WHERE TABLE_SCHEMA=database()';
        const columns = await new Access(body).setCtx(t.ctx).query(sql2);

        for (let table of tables) {
            table.COLUMNS = columns.filter(n => n.TABLE_NAME === table.TABLE_NAME);
        }

        return tables;
    }

    // 构建文件
    async $build(body) {
        const t = this;
        const tables = await t.getSchema(body);

        // 获取模板
        const dir = t.root;
        for (let item of map) {
            const str = fs.readFileSync(dir + '/' + item.file).toString();

            try {
                // 单文件模式
                if (item.single) {
                    const code = coder(str, { tables: tables });
                    const name = item.target;
                    t.save(t.root + '/' + name, code);
                    continue;
                }

                // 多文件模式
                for (let table of tables) {
                    const code = coder(str, { table: table });
                    const name = item.target.replace(/{TABLE_NAME}/ig, table.TABLE_NAME);
                    t.save(t.root + '/' + name, code);
                }
            } catch (err) {
                return {
                    item: item,
                    message: err.message,
                    stack: err.stack
                }
            }
        }

        return { tables: tables, files: t.files };
    }

}