'use strict';

/**
 * 文件读写操作 by yjf
 * 2019-5-23
 */

// #region [import]

const fs = require('fs');
const path = require('path');

// #endregion

module.exports = class {

    // 读取文件
    $readFile(body) {
        const path = this.getPath(body);
        const data = fs.readFileSync(path).toString();

        return { data: data };
    }

    // 写入文件
    $writeFile(body) {
        const path = this.getPath(body);
        const data = body.code;

        fs.writeFileSync(path, data);
        this.clearCache(path);

        return { success: true };
    }

    // 删除文件
    $removeFile(body) {
        const path = this.getPath(body);

        fs.unlinkSync(path);
        this.clearCache(path);

        return { success: true };
    }

    // 创建目录或文件
    $create(body) {
        const path = this.getPath(body);

        if (path.endsWith('/')) {
            fs.mkdirSync(path);
        } else {
            fs.writeFileSync(path, '//' + new Date());
            this.clearCache(path);
        }

        return { success: true };
    }

    // 读取目录
    $readDir(body) {
        const path = this.getPath(body) + '/';
        const dirs = [];
        const files = [];

        const array = fs.readdirSync(path);

        for (let name of array) {
            const info = fs.lstatSync(path + name);
            if (info.isDirectory()) {
                dirs.push(name);
            } else {
                files.push(name);
            }
        }

        return { dirs: dirs, files: files };
    }

    // 删除目录
    $removeDir(body) {

        const path = this.getPath(body) + '/';
        fs.rmdirSync(path);

        return { success: true };
    }

    // 获取路径
    getPath(body) {
        const ctx = this.ctx;
        return path.resolve(ctx.env.root + body.path);
    }

    // 清理缓存
    clearCache(path) {
        if (require.cache[path]) {
            require.cache[path] = null;
        }
    }
}