
xf.module({

    name: 'DEditor.Grid',

    using: [
        'css!../tkui.css'
    ],

    module: {

        init: function (page) {
            var el = page.context.el;
            var list = el.classList;
            list.add('tk-view');
            list.add('tk-grid');
        },

        list: {

            static: {
                defaultCondition: {},     // 默认查询条件

                // 事件触发的元素包含disabled类
                isDisabled: function () {
                    var e = window.event;
                    if (!e) { return false; }

                    var clist = e.target.classList;
                    return clist.contains('disabled') || clist.contains('locked');
                }
            },

            data: {
                condition: {},           // 查询条件
                sortField: '',           // 排序依据
                sortDesc: false,         // 倒叙排列
                filterKey: '',           // 过滤关键字

                searchDesc: '',         // 搜索描述
                metaMap: {},            // 扩展的元数据

                tableName: '',           // 表名
                tableComment: '',         // 表描述
                idField: '',             // ID字段
                fields: [],              // 字段
                items: [],               // 记录
                selected: [],            // 选中
                singleSelect: true,      // 单选模式

                pagerIndex_: 0,          // 当前页数
                pagerSize: 0,            // 每页数量
                pagerCount: 0,           // 总页数
                pagerTotal: 0,           // 总条数

                pickMode: 0,             // 1-单选, 2-多选
                showMode: 'default',      // 显示模式 default-自适应, definded-自定义
                tableStyle: 'classic'    //simple/classic/fashion
            },

            computed: {

                // 模式
                mode: function () {
                    var para = this.Context.para || {};
                    return para.mode;
                },

                // 可见字段
                listFields: function () {
                    return this.fields.filter(function (n) {
                        return n.show;
                    });
                },

                // 选中的ID
                selectedID: function () {
                    var t = this;
                    if (t.selected.length !== 1) { return null; }
                    return t.filterItems[t.selected[0]][t.idField];
                },

                // 选中的ID数组
                selectedIDs: function () {
                    var t = this;
                    var ids = [];
                    for (var i = 0; i < t.selected.length; i++) {
                        ids.push(t.filterItems[t.selected[i]][t.idField]);
                    }
                    return ids;
                },

                // 选中的行
                selectedRow: function () {
                    var t = this;
                    if (t.selected.length !== 1) { return null; }
                    return t.filterItems[t.selected[0]];
                },

                // 选中的行数组
                selectedRows: function () {
                    var t = this;
                    var rows = [];
                    for (var i = 0; i < t.selected.length; i++) {
                        rows.push(t.filterItems[t.selected[i]]);
                    }
                    return rows;
                },

                // 过滤后的记录
                filterItems: function () {
                    var t = this;
                    var key = t.filterKey;
                    if (key.length === 0) {
                        return t.items;
                    }
                    var arr = t.items.filter(function (n) {
                        return n.Title.indexOf(key) > -1;
                    });

                    return arr;
                },

                // 当前页数
                pagerIndex: {
                    get: function () {
                        return parseInt(this.pagerIndex_);
                    },
                    set: function (v) {
                        var t = this;

                        if (isNaN(parseInt(v))) { v = 1; }
                        if (v < 1) { v = 1; }
                        if (v > t.pagerCount) { v = t.pagerCount; }

                        this.pagerIndex_ = v;
                    }
                },

                // 表格宽度
                tableWidth: function () {
                    var t = this;
                    if (t.showMode === 'default') {
                        return '100%';
                    }

                    var width = 100;
                    for (var i = 0; i < t.listFields.length; i++) {
                        width += parseInt(t.listFields[i].width);
                    }

                    return width + 'px';
                },

                // 关联按钮可见
                relateSeen: function () {
                    var t = this;
                    var map = window.table_relate_map;
                    if (map && map[t.tableName]) {
                        return true;
                    }
                    return false;
                }
            },

            watch: {
                singleSelect: function (v) {
                    var t = this;
                    var len = t.selected.length;
                    if (!v || len < 2) { return; }

                    t.selected = [t.selected[0]];
                }
            },

            methods: {

                // 创建按钮（不选可用）
                create: function () {
                    var t = this;
                    if (t.isDisabled()) { return; }

                    t.Page.go(t.editViewName, {
                        mode: 1,
                        idField: t.idField,
                        fields: t.fields,
                        metaMap: t.metaMap,
                        editInputUnlock: t.editInputUnlock,
                        callback: function () { t.search(); }
                    });
                },

                // 修改按钮（单选可用）
                update: function () {
                    var t = this;
                    if (t.isDisabled()) { return; }
                    var para = {
                        id: t.selectedID,
                        mode: 2,
                        idField: t.idField,
                        fields: t.fields,
                        metaMap: t.metaMap,
                        editInputUnlock: t.editInputUnlock,
                        callback: function () { t.search(); }
                    };
                    t.Page.go(t.editViewName, para);
                },

                // 删除按钮（多选可用）
                remove: function () {
                    var t = this;
                    if (t.isDisabled()) { return; }

                    var ids = t.selectedIDs || [];
                    if (!confirm('注意：删除后无法恢复!\n确定要删除 ' + ids.length + ' 条数据吗?')) { return; }

                    var code = Math.random().toString().substr(2, 4);
                    var str = prompt('输入验证码：' + code);
                    if (!str) { return; }
                    if (str !== code) { return alert('验证码错误'); }

                    t.Modal
                        .post('@remove', { ids: ids.join(',') })
                        .done(function (result) {
                            alert('删除了 ' + result + ' 条数据');
                            t.search();
                        });
                },

                // 刷新（不选可用）
                refresh: function () {
                    var t = this;
                    if (t.isDisabled()) { return; }

                    t.search();
                },

                // 关系（单选可用）
                relate: function () {
                    const t = this;
                    t.Page.go('views.grid-relate', {
                        tableName: t.tableName,
                        tableComment: t.tableComment,
                        row: t.selectedRow,
                        fields: t.fields
                    }/*, { animate: 'bottom' }*/);
                },

                // 根据选定数量判断是否可用
                isEnable: function (p) {
                    var t = this;
                    var len = t.selected.length;
                    var all = t.items.length;
                    switch (p) {
                        case 0: return len === 0;         // 无选中
                        case 1: return len === 1;         // 选中一条
                        case 2: return len > 0;           // 选中多条
                        case 10: return all > 0 && len < t.items.length;   // 可全选
                        case 11: return all > 0 && len > 0 && len < t.items.length;    //可以反选
                        default: return true;
                    }
                },

                // 是否被选中
                isSelected: function (i) {
                    var t = this;
                    return t.selected.indexOf(i) >= 0;
                },

                // 滚动表格
                tableScroll: function (e) {
                    var top = e.target.scrollTop + 'px';
                    var ths = this.$el.querySelectorAll('th');
                    for (var i = 0; i < ths.length; i++) {
                        ths[i].style.top = top;
                    }
                },

                // 列表选择
                select: function (p) {
                    var t = this;
                    var i = 0;

                    if (t.isDisabled()) { return; }

                    // 点选
                    if (typeof p === 'number') {
                        var index = t.selected.indexOf(p);
                        if (index > -1) {
                            t.selected.splice(index, 1);
                        } else if (t.singleSelect) {
                            t.selected = [p];
                        } else {
                            t.selected.push(p);
                        }
                    }

                    // 全选
                    else if (p === 'all') {
                        t.selected = [];
                        for (i = 0; i < t.items.length; i++) { t.selected.push(i); }
                    }

                    // 反选
                    else if (p === 'invert') {
                        var selected = [];
                        for (i = 0; i < t.items.length; i++) {
                            if (t.selected.indexOf(i) < 0) { selected.push(i); }
                        }
                        t.selected = selected;
                    }

                    // 清空
                    else if (p === 'clear') {
                        t.selected = [];
                    }

                    // 切换到多选
                    if (t.selected.length > 1) {
                        t.singleSelect = false;
                    }
                },

                // 列表选定
                pick: function (p) {
                    var t = this;
                    // 点选
                    if (typeof p === 'number') {
                        var index = t.selected.indexOf(p);
                        if (index > -1) {
                            t.selected.splice(index, 1);
                        } else {
                            t.selected.push(p);
                        }
                    }

                    if (t.pickMode === 1 && t.selected.length === 1) {
                        var fn = t.Context.para.callback;
                        var row = t.selectedRow;
                        if (fn(row)) { t.Page.close(); }
                    }
                },

                // 翻页
                pager: function (p) {
                    var t = this;
                    if (t.isDisabled()) { return; }

                    var index = t.pagerIndex;
                    if (typeof p === 'number') {
                        t.pagerIndex = p;
                    } else if (p === 'set') {
                        var v = prompt('设置页码', t.pagerIndex);
                        v && (t.pagerIndex = v);
                    } else if (p === 'prev') {
                        t.pagerIndex--;
                    } else if (p === 'next') {
                        t.pagerIndex++;
                    } else if (p === 'setting') {
                        t.Page.go('views.grid-cfg', { list: t }/*, { animate: 'bottom' }*/);
                    }

                    if (index !== t.pagerIndex) {
                        t.search();
                    }
                },

                // 查询数据
                search: function (reset, opt) {
                    var t = this;
                    opt = opt || {};

                    var Search = t.Modal.search;
                    if (Search) {
                        Search.close();
                    }

                    t.selected = [];
                    if (reset) { t.pagerIndex_ = 1; }

                    var api = opt.api || t.Modal.map.listSearchApi || '@search';
                    var para = {
                        condition: t.condition,
                        sortField: t.sortField,
                        sortDesc: t.sortDesc,
                        needTotal: reset || false,
                        pagerIndex: t.pagerIndex,
                        pagerSize: t.pagerSize
                    };

                    t.Modal.post(api, para, { postJSON: true }).done(function (result) {
                        t.$el.querySelector('.tk-table').scrollTop = 0;
                        t.items = result.items;

                        if (result.total >= 0) {
                            t.pagerTotal = result.total;
                            t.pagerCount = Math.ceil(t.pagerTotal / t.pagerSize);
                        }
                    });
                },

                // 查询元数据
                loadMeta: function (ready, opt) {
                    var t = this;
                    opt = opt || {};

                    var api = opt.api || '@schema';
                    var para = opt.para || {};

                    t.Modal.post(api, para).done(function (result) {

                        var List = t.Modal.list;
                        var Search = t.Modal.search;

                        // 初始化列表
                        List.sortField = result.sortField;
                        List.sortDesc = result.sortDesc;
                        List.pagerSize = result.pagerSize;
                        List.tableName = result.tableName;
                        List.tableComment = result.tableComment;
                        List.idField = result.idField;
                        List.fields = result.fields;

                        // 字段太多时使用自定义宽度
                        if (List.listFields.length > 5) {
                            List.showMode = 'definded';
                        }

                        var condition = {};
                        if (Search) {
                            condition = xf(condition, JSON.copy(Search.condition), result.condition);
                        } else {
                            condition = xf(condition, result.condition);
                        }

                        List.defaultCondition = JSON.copy(condition);
                        List.condition = JSON.copy(condition);

                        // 扩展数据
                        List.metaMap = result.metaMap;
                        ready(result);
                    });
                },

                // td的值
                tdValue: function (row, field, i, j) {
                    var t = this;
                    var fn = t['#' + field];        // 定义一个#开头的方法，就能重设字段值
                    if (typeof fn === 'function') {
                        return fn.call(t, row, field, i, j);
                    } else {
                        var v = row[field];

                        // 处理时间
                        var f = t.fields.find(n => n.field === field);
                        if (f && /time|date/i.test(f.type)) {
                            v = xf.date(new Date(v), 'yyyy-MM-dd HH:mm:ss')
                        }

                        if (typeof v !== 'string') { return v; }

                        // 处理多行文本
                        if (v.indexOf('\n') > -1) { v = v.split('\n')[0] + '…'; }
                        if (v.length > 100) { v = v.substr(0, 100) + '…'; }
                        return xf.el('div', { innerText: v }).innerHTML;
                    }
                },

                // 打开搜索
                openSearch: function () {
                    this.$el.classList.add('is-blur');
                    var Search = this.Modal.search;
                    Search.reset();
                    Search.seen = true;
                    setTimeout(function () { Search.open = true; }, 80);
                },

                // 打开或关闭按钮组
                toggleButtons: function () {
                    var t = this;
                    var topbar = t.$el.querySelector('.tk-head');
                    topbar.classList.toggle('show');
                },

                // 用于小屏幕设备的行描述
                rowDescription: function (row) {
                    return JSON.stringify(row);
                },

                // 选择并返回
                selectBack: function (e) {
                    var t = this;
                    if (e === null) {
                        t.Modal.callback(null);
                        t.Page.back();
                        return;
                    }
                    var row = t.selectedRow;
                    if (row) {
                        t.Modal.callback(row);
                        t.Page.back();
                    }
                }
            },

            directives: {
                'body-set': function (el, binding, vnode) {
                    var t = vnode.context;
                    if (t.selected.length > 0) {
                        el.classList.add('is-selected');
                    } else {
                        el.classList.remove('is-selected');
                    }
                },

                'list-btn': {
                    inserted: function (el) {
                        var clist = el.classList;
                        clist.add('list-btn');
                        clist.add('disabled');
                    },
                    update: function (el, binding, vnode) {
                        var t = vnode.context;
                        var len = t.selected.length;
                        var all = t.items.length;
                        var val = binding.value;
                        var enable = true;

                        switch (val) {
                            case true: enable = true; break;
                            case false: enable = false; break;
                            case 0: enable = len === 0; break;         // 无选中
                            case 1: enable = len === 1; break;         // 选中一条
                            case 2: enable = len > 0; break;          // 选中多条
                            case 10: enable = all > 0 && len < t.items.length; break;   // 可全选
                            case 11: enable = all > 0 && len > 0 && len < t.items.length; break;    //可以反选
                        }

                        if (enable) {
                            el.classList.remove('disabled');
                        } else {
                            el.classList.add('disabled');
                        }
                    }
                }
            }
        },

        tree: {
            methods: {

                toggleItem: function (v) {
                    //var t = this;
                    //Vue.nextTick(function () {
                    //    t.resetSelected();
                    //});
                },

                selectItem: function (v) {
                    var t = this;
                    v.on = !v.on;
                    t.resetSelected();
                },

                resetSelected: function () {
                    var t = this;
                    var result = [];
                    var fn = function (n) {
                        if (n.on) { result.push(n.item); }

                        var arr = n.$children || [];
                        for (var i = 0; i < arr.length; i++) {
                            fn(arr[i]);
                        }
                    };
                    fn(t.$refs.tree);
                    t.selected = result;
                },

                findChildren: function (v) {
                    var result = [];
                    var fn = function (n) {
                        result.push(n);

                        var arr = n.$children || [];
                        for (var i = 0; i < arr.length; i++) {
                            fn(arr[i]);
                        }
                    };
                    fn(v);
                    return result;
                }
            },

            // 创建树形节点的组件
            $createItem: function (tag, template) {
                var result = {
                    template: template,
                    components: {},
                    props: { item: null },
                    data: function () {
                        return {
                            open: false,
                            on: false
                        };
                    },
                    computed: {
                        isFolder: function () {
                            var t = this;
                            return t.item.children && t.item.children.length;
                        },
                        hdClass: function () {
                            var t = this;
                            if (!t.isFolder) { return 'hd hd-0'; }
                            if (!t.open) { return 'hd hd-1'; }
                            return 'hd hd-2';
                        }
                    },
                    methods: {
                        toggle: function () {
                            var t = this;
                            if (t.isFolder) {
                                t.open = !t.open;
                                t.$emit('toggle', t);
                            }
                        },
                        ontoggle: function (e) {
                            var t = this;
                            t.$emit('toggle', e);
                        },
                        select: function () {
                            var t = this;
                            t.$emit('select', t);
                        },
                        onselect: function (e) {
                            var t = this;
                            t.$emit('select', e);
                        }
                    }
                };

                result.components[tag] = result;
                return result;
            },

            // 列表转换到树的子节点
            $arr2Children: function (root, list, idName, parentName) {

                var fn = function (target) {
                    var children = list.filter(function (n) {
                        return n[parentName] === target[idName];
                    });

                    if (children.length > 0) {
                        target.children = children;
                        for (var i = 0; i < children.length; i++) {
                            fn(children[i]);
                        }
                    }
                };

                return fn(root);
            },

            // 每一个节点
            $each: function (tree, fn) {
                var t = this;
                var f = function (cm) {
                    fn(cm);

                    var children = cm.$children || [];
                    for (var i = 0; i < children.length; i++) {
                        f(children[i]);
                    }
                };
                f(tree);

            }
        },

        search: {

            data: {
                seen: false,
                open: false,
                condition: {}
            },

            methods: {

                // 推测题目
                getName: function (name) {
                    var t = this;
                    var arr = t.Modal.list.fields;
                    var field = arr.filter(function (n) {
                        return n.field === name;
                    });
                    return field[0] ? field[0].name : name;
                },

                // 确认
                search: function () {
                    var t = this;
                    var msg = t.Modal.checker(t.condition, t.checker, {
                        box: t.$el,
                        notNull: false
                    });
                    if (msg) { return; }

                    t.Modal.list.condition = JSON.copy(t.condition);
                    t.Modal.list.search(true);
                    t.Modal.list.searchDesc = t.getDescription();
                },

                // 取消
                reset: function () {
                    var t = this;
                    t.condition = JSON.copy(t.Modal.list.condition);
                },

                // 清空
                clear: function () {
                    var t = this;
                    var List = t.Modal.list;
                    List.condition = JSON.copy(List.defaultCondition);
                    t.Modal.list.search(true);
                    t.condition = List.defaultCondition;
                    t.Modal.list.searchDesc = t.getDescription();
                },

                // 关闭
                close: function () {
                    var t = this;
                    t.reset();
                    t.open = false;
                    t.Modal.list.$el.classList.remove('is-blur');

                    // 去掉错误提示
                    var els = t.$el.querySelectorAll('.error');
                    if (els) {
                        var len = els.length;
                        for (var i = 0; i < len; i++) {
                            var el = els[i];
                            el.classList.remove('error');
                            el._checker_msg && el._checker_msg.remove();
                        }
                    }

                    setTimeout(function () {
                        t.seen = false;
                    }, 100);
                },

                // 获取搜索参数的描述
                getDescription: function () {
                    var t = this;
                    var desc = [];
                    for (var k in t.condition) {
                        var v = t.condition[k];
                        if (v !== '') {
                            desc.push(t.getName(k) + ': ' + v);
                        }
                    }

                    if (desc.length === 0) {
                        return null;
                    }

                    return desc.join(', ');
                }
            },

            computed: {
                metaMap: function () {
                    var t = this;
                    return t.Modal.list.metaMap;
                }
            },

            directives: {
                'body-set': function (el, binding, vnode) {
                    var t = vnode.context;
                    if (t.seen) {
                        el.classList.add('is-open');
                    } else {
                        el.classList.remove('is-open');
                    }

                    if (t.open) {
                        el.classList.add('is-ready');
                    } else {
                        el.classList.remove('is-ready');
                    }
                }
            }
        },

        listExt: {
            methods: {
                setMark: function () {
                    var t = this;
                    if (t.isDisabled()) { return; }

                    var btns = [];
                    var marks = t.metaMap.markType;
                    for (var i = 0; i < marks.length; i++) {
                        var mark = marks[i];
                        btns.push({ text: '标记为：' + mark.key, value: mark.value });
                    }

                    t.Page.go('@Admin.ButtonList', {
                        title: '设置标记',
                        btns: btns,
                        callback: function (btn) {
                            t.Page.back();
                            this.opt.controller = t.Modal.opt.controller;
                            this.post('@setMarkType', {
                                ids: t.selectedIDs,
                                val: btn.value
                            }).done(function (result) {
                                t.search();
                            });
                        }
                    }/*, { animate: 'bottom' }*/);
                },

                btnunlock: function () {
                    var btns = this.$el.querySelectorAll('.list-btn.locked');
                    for (var i = 0; i < btns.length; i++) {
                        btns[i].classList.remove('locked');
                    }
                },

                editunlock: function () {
                    this.editInputUnlock = true;
                }
            }
        }
    }
});