xf.module({

    name: 'builders.tkui',

    module: function () {

        var Builder = function (context, opt) {

            var t = this;
            var c = context;

            // 成员
            t.context = c; // 上下文
            t.name = c.name; // 视图名
            t.opt = opt = opt || {}; // 构建参数
            t.map = opt.map || {}; // 自定义配置
            c.modal = t;

            // 设置题目
            if (opt.title) {
                context.view.title = opt.title;
            }

            // 直接访问时跳转到宿主页
            if (opt.host && c.para === null) {
                location.href = '?name=' + opt.host;
                return;
            }

            // 载入之前的钩子
            if (opt.beforeLoad && opt.beforeLoad() === false) {
                return;
            }

            // 构建元素
            c.el = xf.el('div', {
                innerHTML: c.html,
                className: t.name.replace(/\./g, '-')
            });
            c.view.ready(c);
            delete c.html;

            // 初始化指定的模块
            var modules = opt.initModules;
            if (modules) {
                for (var i = 0; i < modules.length; i++) {
                    modules[i].init(t);
                }
            }
        };

        Builder.prototype = {

            // 创建Vue对象
            vue: function (name) {

                var t = this;
                var el = t.context.el;
                if (!el) { throw '没有找到context.el'; }

                // 初始化Vue对象配置
                var members = {};
                var opt = {
                    el: el.querySelector('.' + name),
                    data: {},
                    computed: {
                        Page: function () { return t.context.page; }, // 页面对象
                        View: function () { return t.context.view; }, // 视图对象
                        Context: function () { return t.context; }, // 上下文
                        Modal: function () { return t; } // 视图控制器
                    },
                    methods: {

                    }
                };

                // 扩展配置
                var arr = xf.toArr(arguments);
                for (var i = 1; i < arr.length; i++) {
                    var item = arr[i];
                    for (var k in item) {
                        var value = item[k];
                        // 直接附加的静态对象
                        if (k === 'static' || k === 'members') {
                            xf(members, value);
                            continue;
                        }
                        if (k[0] === '$' || typeof value !== 'object') {
                            continue;
                        }
                        if (opt[k] === undefined) { opt[k] = {}; }
                        xf(opt[k], value);
                    }
                }

                // 创建对象并相互绑定
                var v = new Vue(opt);
                for (var m in members) {
                    v[m] = members[m];
                }

                return t[name] = v;
            },

            // 视图控制器创建就绪
            ready: function (fn) {
                var t = this;
                var c = t.context;
                if (!c.el) { return; }

                // 执行回调
                fn && fn.call(t, c.para);
            },

            // 提交数据
            post: function (url, data, option) {
                var t = this;

                var opt = {};

                opt.controller = t.opt.controller;

                opt.beforeSend = function () {
                    t._postRemain = t._postRemain || 0;
                    t._postRemain++;
                    t.context.view.els.body.classList.add('xv-loading');
                };

                opt.beforeReady = function () {
                    t._postRemain--;
                    if (t._postRemain === 0) {
                        t.context.view.els.body.classList.remove('xv-loading');
                    }
                };

                return new xf.Post(opt).post(url, data, option);
            },

            // 检查数据
            checker: function (row, rules, opt) {
                var t = this;
                opt = opt || {};

                // 在元素上显示错误
                var tip = function (key, msg) {
                    var el = opt.box.querySelector('[data-checker="' + key + '"]');
                    if (!el) return false;

                    el.classList.add('error');

                    // 第一次创建
                    if (!el._checker_msg) {
                        el._checker_msg = xf.el('div', { className: 'checker-msg' });
                        el.addEventListener('click', function (e) {
                            e.target.classList.remove('error');
                            el._checker_msg.remove();
                        }, false);
                    }

                    el._checker_msg.innerHTML = msg;
                    el.after(el._checker_msg);

                    return true;
                };

                // 根据规则调整
                var result = null;

                for (var key in rules) {
                    var val = row[key];
                    if (val === undefined) { continue; }

                    // 将规则转为数组
                    var rule = rules[key];
                    if (!(rule instanceof Array)) { rule = [rule]; }

                    // 依次判定
                    for (var i = 0; i < rule.length; i++) {
                        var r = rule[i];

                        if (opt.notNull === false && val === '') {
                            continue;
                        }

                        var ok = r.rgx.test(val);
                        if (ok) { continue; }

                        result = result || [];
                        tip(key, r.msg) || result.push(r.msg);
                        break;
                    }
                }

                return result;
            },

            // 回调函数
            callback: function (para) {
                xf.log('[#93B983]callback', para);
                return this.context.para.callback.call(this, para);
            },

            // 到其他页面选取信息
            goSelect: function (para, map, key, e) {
                var t = this;
                var page = t.context.page;
                var sec = para.split(/\#|\.|\:/g);
                var name = '@' + sec[1] + '-grid';

                // 尝试去掉错误信息
                try {
                    e.target.parentNode.querySelector('*[data-checker]')._checker_msg.remove();
                } catch (e) {
                }

                page.go(name, {
                    mode: 'select',
                    callback: function (item) {
                        map[key] = item === null ? '' : item[sec[2]];
                    }
                });
            },

            // 设置外键信息
            setOuterExp: function (exp, v, map, key) {
                if (v === undefined || v == '') {
                    return map[key] = '-- 请选择 --';
                } else {
                    map[key] = '......'
                }

                const t = this;
                const arr = exp.split(/\#|\.|\:/g); //#th_user.id:nick_name
                const url = '@../' + arr[1] + '/find';
                const para = {}
                para[arr[2]] = v;
                t.context.modal.post(url, para).done(function (result) {
                    map[key] = result ? result[arr[3]] : '[ null ]';
                });

            },

            // 切换外键显示模式
            toggleOuter: function (e) {
                var target = e.target;
                var els = target.parentNode.querySelectorAll('[data-toggle-show]');
                for (var el of els) {
                    var att = el.getAttribute('data-toggle-show');
                    var val = att === 'true' ? 'false' : 'true';
                    el.setAttribute('data-toggle-show', val);
                }
            },

            // 去日期选择器
            goDtpicker: function (row, key) {
                var t = this;
                var page = t.context.page;
                page.go('views.dtpicker', {
                    value: row[key],
                    callback: function (date) {
                        row[key] = xf.date(date, 'yyyy-MM-dd HH:mm:ss');
                    }
                }, { width: '440px', height: '440px', animate: 'bottom' });
            }
        };

        // 在数组中，根据value找key
        Builder.findKey = function (arr, val) {
            arr = arr || [];
            var len = arr.length;
            for (var i = 0; i < len; i++) {
                var item = arr[i];
                if (item.value.toString() === val.toString()) {
                    return item.key;
                }
            }
            return undefined;
        };

        return Builder;
    }
});