
!function () {

    // #region static
    var PX = 'px',

        C_VIEW = 'xv-view',             /* 视图模块的类名 */
        C_MASK = 'xv-mask',             /* 视图蒙版类名 */
        C_MAIN = 'xv-main',             /* 视图主体类名 */
        C_BODY = 'xv-body',             /* 视图内容类名 */
        C_BORDER = 'xv-border',         /* 视图边框的类名前缀 */
        C_RESIZER = 'xv-resizer',       /* 调整大小时拖拽框的类名 */
        C_READY = 'xv-ready',           /* 就绪 */

        C_HEAD = 'xv-head',             /* 标题栏 */
        C_TITLE = 'xv-title',           /* 题目 */
        C_CLOSE = 'xv-close',           /* 关闭 */

        // 状态
        C_HOME = 'xv-home',             /* 主要视图的类名 */
        C_FOCUS = 'xv-focus',           /* 焦点视图的类名 */
        C_LOADING = 'xv-loading',       /* 载入中的类名 */
        C_RESIZE = 'xv-resize',         /* 调整大小时的类名 */
        C_PREVIEW = 'xv-preview',       /* 预览时的类名 */
        C_IN_PC = 'xv-in-pc',           /* PC布局模式 */
        C_IN_PHONE = 'xv-in-phone'      /* 手机布局模式 */

    var $w = window;

    // 配置
    var $c = function () {
        return xf.config;
    };

    // 元素操作
    var $el = function (el, pro, parent, _) {

        el = typeof el !== 'string' ? el : document.createElement(el);
        if (pro) { for (_ in pro) { el[_] = pro[_]; } }
        if (parent) { parent.appendChild(el); }

        return el;
    };

    // #endregion

    // #region 页面-Page

    var Page = function (para) {

        var t = this;
        t.index = 0;            // 视图编号
        t.history = [];         // 视图栈
        t.bindPopState();       // 监视弹出

        // 附加配置
        xf(xf.config, para);

        // 配置requirejs
        xf.require.config($c().require.config());
        xf.require.onError = $c().require.onError;

        // 配置builder
        xf.builder = xf.config.builder;
    };

    Page.prototype = {

        // 打开视图
        open: function (name, para, opt) {
            var t = this;
            opt = opt || {};

            DLog('[#93B983]open', name, xf.data(para));

            // 创建并载入视图
            var view = new View(opt, t);
            var url = $c().view_name2url(name);
            return view.load(url, para);
        },

        // 关闭视图
        close: function () {
            var n = document.getElementsByClassName(C_FOCUS)[0];
            if (!n) { return; }
            if (n.classList.contains(C_HOME)) { return; }

            n.DViewTarget.remove(300);
        },

        // 进入视图
        go: function (name, para, opt) {
            var t = this;
            var key = t.index++ + name;

            t.history.push(key);
            $w.history.pushState({ key: key }, null, '?name=' + name);

            return t.open(name, para, opt);
        },

        // 替换视图
        goset: function (name, para, opt) {
            var t = this;
            var key = t.index++ + name;
            var len = t.history.length || 1;

            t.history[len - 1] = key;
            $w.history.replaceState({ key: key }, null, '?name=' + name);

            return t.open(name, para, opt);
        },

        // 后退
        back: function () {
            history.back();
        },

        // 监视popstate事件
        bindPopState: function () {
            var t = this;
            $w.addEventListener('popstate', function (e) {

                // 目标页不在历史中
                var index = t.history.indexOf(e.state.key);
                if (index < 0) { return history.back(); }

                // 目标页是历史页(不是当前页)
                if (index < t.history.length - 1) {
                    t.history.length = index + 1;
                    t.close();
                }
            }, false);
        }
    };

    // #endregion

    // #region 视图-View

    var View = function (opt, page) {
        var t = this;

        // 初始化
        t._initOption(opt);                   // 初始化配置
        t.els = t._createElements();          // 创建元素
        t.parent = document.body;             // 挂载的父元素
        t.context = {                         // 上下文
            page: page,
            view: t
        };

        t.oncreated(t);
        t.parent.appendChild(t.els.view);    // 挂载元素

        // 监听窗口变化
        window.addEventListener('resize', function () {
            t._setDeviceName();
        });

        setTimeout(function () {
            t.focus();
        }, 100);
    };

    View.prototype = {

        // 载入视图瞎的资源
        load: function (url, para) {
            var t = this;
            t.context.para = para;

            var paths = t._createPaths(url);
            xf.require(paths, function (builder, html) {
                t._buildSource(builder, html);
            });

            return t;
        },

        // 简单的构建
        build: function (title) {
            var t = this;
            var c = t.context;

            t.title = title;
            t.body = xf.el('div', {
                innerHTML: c.html,
                className: c.name
            });

            t.ready();
        },

        // 构建目标资源就绪
        ready: function () {
            var t = this;

            // 装载目标元素
            var els = t.els;
            var el = t.context.el;
            els.body.appendChild(el);
            els.view.appendChild(els.main);
            els.view.classList.remove(C_LOADING);

            // 效果和样式
            var main = els.main;
            t.animate && setTimeout(function () {
                main.classList.remove(t.animate);
            }, 100);                          // 入场效果
            var style = main.style;
            if ($w.innerWidth > t.minWidth) {
                t.width && (style.width = t.width);       // 宽
                t.height && (style.height = t.height);     // 高
            } else {
                style.width = '100%';
                style.height = '100%';
            }

            t.useDrag && t.setDrag();      // 拖拽
            t.useCenter && t.toCenter(main);  // 居中

            // 布局样式
            t._setDeviceName();

            // 重绘以获取焦点:
            setTimeout(function () {
                els.body.classList.add(C_READY);
            }, 500);

        },

        get title() {
            return this.els.title.innerHTML;

        },

        set title(v) {
            this.els.title.innerHTML = v;
        },

        get body() {
            return this.context.el;
        },

        set body(el) {
            this.context.el = el;
        },

        // 设置拖拽!
        setDrag: function (header) {
            var t = this,
                i = 0,
                m = t.els.view,
                c = t.els.main,
                s = c.style,
                d = null,       // drag paras
                r = null,       // resizing
                n = null,       // now
                a = [
                    header || t.els.header,
                    $el('div', { DV_border: 'l', className: C_BORDER + '-l' }, c),
                    $el('div', { DV_border: 'r', className: C_BORDER + '-r' }, c),
                    $el('div', { DV_border: 'b', className: C_BORDER + '-b' }, c),
                    $el('div', { DV_border: 'lb', className: C_BORDER + '-lb' }, c),
                    $el('div', { DV_border: 'rb', className: C_BORDER + '-rb' }, c)
                ];

            // mousedown
            var D = function (e) {

                if (t._size) { return; }
                if (e.target.tagName === 'A') { return; }

                d = $w.getComputedStyle(c);
                d = {
                    x: e.x,
                    y: e.y,
                    t: parseInt(d.top),
                    l: parseInt(d.left),
                    w: parseInt(d.width),
                    h: parseInt(d.height),
                    c: e.target.DV_border || 'h'
                };

                n = {
                    t: d.t,
                    l: d.l,
                    w: d.w,
                    h: d.h
                };

                m.addEventListener('mousemove', M);
                m.addEventListener('mouseup', U);
                e.stopPropagation();
            };

            // mousemove
            var M = function (e) {

                var x = e.x - d.x,
                    y = e.y - d.y;

                switch (d.c) {
                    case 'h':
                        n.l = d.l + x;
                        n.t = d.t + y;
                        break;
                    case 'l':
                        n.l = d.l + x;
                        n.w = d.w - x;
                        break;
                    case 'r':
                        n.w = d.w + x;
                        break;
                    case 'b':
                        n.h = d.h + y;
                        break;
                    case 'lb':
                        n.l = d.l + x;
                        n.w = d.w - x;
                        n.h = d.h + y;
                        break;
                    case 'rb':
                        n.w = d.w + x;
                        n.h = d.h + y;
                        break;
                }

                // chrome bug fixed
                if (!x && !y) { return; }

                R(true);
            };

            // mouseup
            var U = function () {

                if (!d) { return; }

                m.removeEventListener('mousemove', M);
                m.removeEventListener('mouseup', U);

                R(false);
                d = n = r = null;
                t._setDeviceName();
            };

            // resizer
            var R = function (p, _) {

                if (!p) {
                    // 完成拖拽
                    m.classList.remove(C_RESIZE);
                    r && m.removeChild(r);

                    // 过界修正
                    var w = $w.innerWidth,
                        h = $w.innerHeight,
                        a = n.l + n.w - w,
                        b = n.t + n.h - h;

                    n.w = Math.min(n.w, w);
                    n.h = Math.min(n.h, h);
                    n.l -= a > 0 ? a : 0;
                    n.t -= b > 0 ? b : 0;

                    n.t < 0 && (n.t = 0);
                    n.l < 0 && (n.l = 0);
                    n.w < t.minWidth && (n.w = t.minWidth);
                    n.h < t.minHeight && (n.h = t.minHeight);

                    t.parent.classList.remove(C_PREVIEW);

                } else if (!r) {
                    // 显示拖拽层
                    m.classList.add(C_RESIZE);
                    r = $el('div', { className: C_RESIZER }, m);

                    t.parent.classList.add(C_PREVIEW);
                }

                p = p ? r.style : s;
                p.top = n.t + PX;
                p.left = n.l + PX;
                p.width = n.w + PX;
                p.height = n.h + PX;
            };

            // bind event
            while (i < 6) {
                var el = a[i++];
                el && el.addEventListener('mousedown', D);
            }

            // dblclick
            if (a[0]) {
                a[0].addEventListener('dblclick', function (e) {

                    if (t._size) {
                        s.width = t._size.w;
                        s.height = t._size.h;
                        t._size = null;
                    } else {
                        t._size = { w: s.width, h: s.height };
                        //s.width = $w.innerWidth - 20 + PX;
                        //s.height = $w.innerHeight - 20 + PX;
                        s.width = '100%';
                        s.height = '100%';
                    }

                    t.toCenter(c);
                    t._setDeviceName();
                });
            }
        },

        // 移动到中心!
        toCenter: function (el) {
            var s = $w.getComputedStyle(el);
            el.style.top = Math.max(0, ($w.innerHeight - parseInt(s.height)) / 3) + PX;
            el.style.left = Math.max(0, ($w.innerWidth - parseInt(s.width)) / 2) + PX;
        },

        // 缩放到尺寸
        toSize: function (width, height, toCenter) {
            var t = this;
            var main = t.els.main;
            width && (main.style.width = width);
            height && (main.style.height = height);
            toCenter && t.toCenter(main);
        },

        // 移除模块
        remove: function (delay) {
            var t = this;
            var fn = function () {
                t.parent.removeChild(t.els.view);
            };

            t._removing = true;

            // 出场动画
            t.animate && t.els.main.classList.add(t.animate);

            delay ? setTimeout(fn, delay) : fn();

            // 设置焦点
            var a = t.parent.getElementsByClassName(C_VIEW);
            var l = a.length;
            while (l--) {
                var m = a[l].DViewTarget;
                if (m._removing) { continue; }

                m.focus();
                return m;
            }
        },

        // 设置模块焦点
        focus: function () {
            var t = this;
            var el = t.parent.getElementsByClassName(C_FOCUS)[0] || null;

            if (el !== t.modal) {
                el && el.classList.remove(C_FOCUS);
                t.els.view.classList.add(C_FOCUS);
                return t;
            }
        },

        // 初始化配置
        _initOption: function (opt) {
            var t = this;
            opt = xf({}, $c().container, opt);

            t.animate = opt.animate;             // 入场动画
            t.useDrag = !!opt.useDrag;           // 启用拖拽
            t.useCenter = !!opt.useCenter;       // 居中
            t.useHeader = !!opt.useHeader;       // 使用头
            t.width = opt.width;
            t.height = opt.height;
            t.minWidth = opt.minWidth;
            t.minHeight = opt.minHeight;
            t.exts = $c().view_exts;
            t.oncreated = opt.oncreated || $c().container.oncreated || function () { };
        },

        // 创建容器元素
        _createElements: function () {
            var t = this;
            var els = {};

            els.view = $el('div', { className: C_VIEW, DViewTarget: t });
            els.mask = $el('div', { className: C_MASK }, els.view);
            els.main = $el('div', { className: C_MAIN });

            // 效果
            els.view.classList.add(C_LOADING);
            t.animate && els.main.classList.add(t.animate);

            // 使用默认头
            if (t.useHeader) {
                els.header = $el('div', { className: C_HEAD }, els.main);
                els.title = $el('div', { className: C_TITLE, innerHTML: '请稍后…' }, els.header);
                els.close = $el('a', { className: C_CLOSE }, els.header);
                els.close.onclick = function () {
                    t.context.page.back();
                    //t.remove();
                };
            }

            els.body = $el('div', { className: C_BODY }, els.main);

            return els;
        },

        // 创建依赖的资源
        _createPaths: function (url) {

            var t = this;
            var paths = [url];

            // 文件夹模式，载入三个文件
            var last = url[url.length - 1];
            if (last === '/') {
                var arr = url.split('/');
                var file = arr[arr.length - 2].replace(/^_*/, '');  // 如果模块名以_开头，文件名去掉开头的_
                var path = url + file;

                paths[0] = t.exts.js.replace('*', path);
                paths[1] = t.exts.doc.replace('*', path);
                paths[2] = t.exts.css.replace('*', path);
            }

            return paths;
        },

        // 构建目标资源
        _buildSource: function (builder, html) {
            var t = this;

            // 发布后的静态资源
            if (builder.STATIC_SRC) {
                html = [
                    builder.STATIC_SRC.html,
                    '<style>\n',
                    builder.STATIC_SRC.css,
                    '</style>'
                ].join('');
            }

            // html中的动态引用
            var rgx = /<xv-include\s+builder=".+"\s*\/>/ig;
            var arr = html.match(rgx);

            // 分析引用地址
            var paths = [];
            if (arr) {
                var ext = t.exts.builderHtml;
                for (var i = 0; i < arr.length; i++) {
                    var name = arr[i].split('"')[1];
                    var src = ext
                        .replace('*', name.replace(/\./g, '/'))
                        .replace('~/', $c().view_root + '/');
                    paths.push(src);
                }
            }

            // 载入引用代码并开始构建
            xf.require(paths, function () {

                // 拼接HTML
                var i = 0;
                var args = xf.toArr(arguments);
                t.context.html = html.replace(rgx, function () { return args[i++]; });

                // 执行方法
                if (typeof builder !== 'function') { return alert(paths[0] + '不是方法'); }
                builder(t.context);
            });
        },

        // 获取布局名(手机/电脑)
        _setDeviceName: function () {
            var t = this;
            var main = t.els.main;
            var style = $w.getComputedStyle(main);
            var width = parseInt(style.width);
            var name = width > t.minWidth ? C_IN_PC : C_IN_PHONE;
            var list = t.els.main.classList;

            list.remove(C_IN_PC, C_IN_PHONE);
            list.add(name);

            t.els.main.classList.add(name);

            return name;
        }
    };

    // #endregion

    // #region 模块-DModule

    var DModule = function (target) {

        // 分析依赖
        var paths = target.using || [];
        for (var i = 0; i < paths.length; i++) {
            paths[i] = paths[i].replace('~/', $c().view_root + '/');
        }

        xf.define(paths, function () {
            var args = xf.toArr(arguments);
            var view = target.view;
            var module = target.module;

            // 构建视图
            if (view) {
                var fn = function (context) {
                    context.name = target.name;
                    view.apply(target, [context].concat(args));
                };
                fn.STATIC_SRC = target.STATIC_SRC;
                return fn;
            }

            // 构建模块
            if (module) {
                return typeof module === 'function' ? module.apply(target, args) : module;
            }
        });
    };

    // #endregion

    // #region 异步请求-DPost

    var DPost = function (opt) {
        var t = this;

        t.opt = opt || {};
        t.xhrs = [];
        t.remain = 0;
        t.fn = null;
        t.next = null;
    };

    DPost.prototype = {

        post: function (name, data, opt) {
            var t = this;
            t.remain++;
            opt = opt || {};

            var logID = xf.newID('logID');
            DLog('[#BBBBBB]post-' + logID, name, xf.data(data));

            var xhr = new XMLHttpRequest();
            var url = $c().api_name2url.call(this.opt, name);
            var body;

            xhr.open('post', url, true);
            if (opt.postJSON) {
                xhr.setRequestHeader('Content-Type', 'application/json');
                body = JSON.stringify(data);
            } else {
                xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
                var _arr = [];
                for (var _k in data) {
                    var _v = data[_k];
                    if (_v === null || _v === undefined) {
                        _v = '';
                    }
                    _arr.push(_k + '=' + encodeURIComponent(_v));
                }
                body = _arr.join('&');
            }

            t.opt.beforeSend && t.opt.beforeSend();

            xhr.send(body);
            xhr.addEventListener('readystatechange', function () {
                if (xhr.readyState !== 4) { return; }

                DLog('[#0C00FF]received-' + logID, name, JSON.parse(xhr.responseText));

                t.opt.beforeReady && t.opt.beforeReady();
                t.remain--;
                t._run();
            });

            t.xhrs.push(xhr);
            return t;
        },

        done: function (fn) {
            var t = this;
            t.fn = fn;

            t.next = new DPost(t.opt);
            return t.next;
        },

        _run: function () {
            var t = this;
            if (!t.fn || t.remain > 0) { return; }

            var args = [];
            var len = t.xhrs.length;
            for (var i = 0; i < len; i++) {
                var arg = $c().api_ready(t.xhrs[i]);
                args.push(arg);

                if (arg === undefined) {
                    //没有返回值，停止继续执行
                    return;
                }
            }
            var result = t.fn.apply(t, args);

            // 设置next
            if (!(result instanceof DPost)) { return; }
            result.fn = t.next.fn;
            result.next = t.next.next;
        }
    };

    // #endregion

    // #region 日志-Log

    var _lenStr = function (v, len) {
        var i = v.length;
        for (; i < len; i++) { v += ' '; }
        return v;
    };

    var DLog = function () {

        var a = xf.toArr(arguments);
        var name = a[0];
        if (typeof name === 'string' && /^\[\#(.+)\]/.test(name)) {
            var m = /^\[(\#.+)\](.+)$/.exec(name);
            if (m) {
                var k = '--- %c[' + m[2] + ']';
                a.splice(0, 1, _lenStr(k, 22), 'color:' + m[1]);

                var b = a[2];
                if (typeof b === 'string') { a[2] = _lenStr(b, 14); }
            }
        }

        //if (!$config.log) { return; }
        //if (app.infos.length > 30) { app.infos.shift(); }
        //app.infos.push(a);

        try { console.log.apply(0, a); }
        catch (e) { console.error(0, e); }
    };

    // #endregion

    xf(xf, {
        el: $el,
        Page: Page,
        View: View,
        Post: DPost,
        module: DModule,
        log: DLog,
    });
}();

