
// 基类
module.exports = class ApiBase {

    /** 构造 */
    constructor(ctx) {
        const t = this;
        t.ctx = ctx;
        t.failMsg = [];
    }

    /** 获取并验证变量 */
    value(val, para, failMsg) {
        const t = this;
        const type = typeof para;
        let result = null;

        switch (type) {

            // 没有限制
            case 'undefined':
                return val;

            // 数字默认值
            case 'number':
                const v = parseInt(val);
                if (!isNaN(v)) { return v; }
                result = para;
                break;

            // 方法验证
            case 'function':
                if (para(val)) { return val; }
                break;

            default:
                // 验证枚举
                if (para instanceof Array) {
                    if (para.indexOf(val) > -1) { return val; }
                }

                // 正则表达式验证
                else if (para instanceof RegExp) {
                    if (val === undefined) { val = ''; }
                    if (para.test(val)) { return val; }
                }
        }

        if (failMsg) {
            t.failMsg.push(failMsg);
        }

        return result;
    }

    /** 判断是否存在失败信息 */
    get isFailed() {
        return this.failMsg.length > 0;
    }

    /** 返回失败信息 */
    fail(msg) {
        const t = this;
        const arr = t.failMsg;

        t.ctx.status = 400;
        if (msg) { arr.push(msg); }

        return { status: 'fail', msg: arr }
    }

    /** 返回错误信息 */
    error(err, msgArr) {
        const t = this;

        t.ctx.status = 500;

        return { status: 'error', err: err || '', msg: msgArr || [] }
    }

    /** 返回成功信息 */
    success(data) {
        return { status: 'success', data: data };
    }

    /** 获取用户ID */
    getUserID() {
        const t = this;
        return parseInt(t.ctx.header['ctx-userid'] || 0);
    }
}