'use strict';
const util = require('util');

// 返回错误
const outError = (ctx, msg, code, detail) => {
    ctx.body = { status: "error", err: msg, msg: detail };
    ctx.status = code;
};

// 获取路由配置
const getRouterOpt = (path, router) => {
    path = path.substr(router.path.length);

    const arr = path.split('/');
    const len = arr.length;
    if (len < 2) { return { aPath: null } }

    path = len > 2 ? (arr.slice(0, len - 2).join('/') + '/') : '';
    let c = arr[len - 2];
    let m = arr[len - 1] || 'index';

    // 是否启用大写模式
    if (router.umode) {
        c = c[0].toUpperCase() + c.substr(1) + '.js';
        m = m[0].toUpperCase() + m.substr(1);
    } else {
        c = c + '.api.js';
        m = '$' + m
    }

    return {
        aPath: router.dir + path + c,
        aName: m
    }
}

// 输出Koa的组件
module.exports = map => {

    return async (ctx, next) => {
        const path = ctx.path;
        const router = map.find(n => path.indexOf(n.path) === 0);

        if (router !== undefined) {

            // 解析
            const { aPath, aName } = getRouterOpt(path, router);
            if (aPath === null) {
                return outError(ctx, `未能解析网址|${path}`, 404);
            }

            // 控制器
            let api;
            try {
                const Api = require(ctx.env.root + aPath);
                api = new Api(ctx, aName);
            } catch (e) {
                return outError(ctx, `控制器错误|${e.message}|${e.stack}`, 500);
            }

            // 方法
            const method = api[aName];
            if (!util.isFunction(method)) {
                return outError(ctx, `没有找到方法|${aName}`, 404);
            }

            // 执行API
            const body = ctx.method === 'GET' ? ctx.query : ctx.request.body;
            try {
                api.ctx = ctx;

                // 前置
                const before = api.before;
                if (util.isFunction(before)) {
                    const result = await before.call(api, aName);
                    if (result === false) { return; }
                }

                // 执行
                ctx.body = await method.call(api, body);

                // 后置
                const after = api.after;
                if (util.isFunction(after)) {
                    await after.call(api, aName);
                }
            } catch (e) {
                return outError(ctx, `执行时出错|${e.message}|${e.stack}`, 500, e.stack.split('\n'));
            }
        }

        await next();
    };
};
