import mysql from 'mysql';
import Select from './select';

/** 表配置 */
interface IConfig {
    sheetName: string,
    primaryKey?: string,
    insertData?: any,
    filter?: string,
    readOnly?: boolean
}

/** 操作结果 */
interface IAffected {
    affectedRows: number
    changedRows: number
    fieldCount: number
    insertId: any
    message: string
    serverStatus: number
    warningCount: number
}

declare class Access {

    db_cfg: any;

    /** 表名 */
    sheetName: string;

    /** 唯一主键 */
    primaryKey: string;

    /** 插入操作的默认值 */
    insertData: Object | undefined;

    /** 查询操作的默认条件 */
    filter: string | undefined;


    /** 构造函数 */
    constructor(db_cfg: any, acc_cfg?: IConfig);


    /** 设置上下文 */
    setCtx(ctx: any): Access;


    /** 设置表名和主键 */
    setEntity(sheetName: string, primaryKey: string): Access;


    /** 追加表名 */
    addEntity(express: string): Access;

    /** 开启事务 */
    beginTransaction(): Promise<any>;

    /** 关闭事务 */
    commit(): Promise<any>;


    /** 执行sql语句 */
    query(sql: string, paras?: any | Array<any>): Promise<any>;


    /** 构造查询
     * @param field 要选择的字段
     * @param where 查询条件
     * @param order 排序规则
     */
    select(field?: string, where?: string, order?: string): Select;


    /** 不依赖过滤器的构造查询
     * @param field 要选择的字段
     * @param where 查询条件
     * @param order 排序规则
     */
    sysSelect(field?: string, where?: string, order?: string): Select;


    /** 添加数据 */
    insert(paras: any): Promise<IAffected>;


    /** 更新数据 */
    update(paras: any, condition?: any): Promise<IAffected>;


    /** 删除数据 */
    delete(condition: string | any): Promise<IAffected>;

    /** 构建外部数据 */
    buildOuter(rows: Array<any>, keys?: Array<string>): Promise<any>;

    /** 附加数据 */
    static withItem(key: string, list: any, outerKey: string, access: Access, where: any): Promise<number>;
}

export = Access;