import Access from './access';

interface IGetPage {
    rows: Array<any>,
    total: number
}

/** 生成SQL的Select */
declare class Select {

    /** 数据表入口 */
    access: Access;


    /** 构造函数 */
    constructor(access: Access, filter?: string);


    /** 添加字段 */
    field(v: string): Select;


    /** 添加条件 */
    where(v: string): Select;


    /** 添加排序 */
    order(v: string): Select;

    /** 添加分组 */
    group(v: string): Select;

    /** 数量和偏移量 */
    limit(count: number, offset?: number): Select;


    /** 构建为sql语句 */
    toSql(): string;


    /** 返回一个值，int或null类型 */
    getInt(paras: any | Array<any>): Promise<number | null>;


    /** 返回一个值 */
    getValue(paras: any | Array<any>): Promise<any | null>;


    /** 返回一条数据 */
    getOne(paras: any | Array<any>): Promise<any | null>;


    /** 返回列表 */
    getList(paras: any | Array<any>): Promise<Array<any>>;


    /** 返回翻页列表 
     * @param paras 参数表
     * @param pageSize 每页数量
     * @param pageIndex 页索引
     */
    getPage(paras: any | Array<any>, pageSize: number, pageIndex: number): Promise<IGetPage>;
}

export = Select;
