

/** 生成SQL的Select
 * 
 */
class Select {

    constructor(access, filter) {
        const t = this;

        t.access = access;
        t._field = [];
        t._where = [];
        t._order = [];
        t._group = [];
        filter && t._where.push(filter);
    }

    /** 添加字段 */
    field(v) {
        v && this._field.push(v);
        return this;
    }

    /** 添加条件 */
    where(v) {
        v && this._where.push(v);
        return this;
    }

    /** 添加排序 */
    order(v) {
        v && this._order.push(v);
        return this;
    }

    group(v) {
        v && this._group.push(v);
        return this;
    }

    /** 数量和偏移量 */
    limit(count, offset) {
        const t = this;

        if (count) {
            t._limit = 'limit ' + (offset ? `${offset},${count}` : count);
        }

        return t;
    }

    /** 构建为sql语句 */
    toSql() {
        const t = this;
        let sql = ['select'];

        // field
        const field = t._field;
        if (field.length === 0) {
            sql.push('*');
        } else {
            sql.push(field.join(','));
        }

        sql.push('from');
        sql.push(t.access.sheetName);

        // where
        const where = t._where;
        if (where.length > 0) {
            sql.push('where');
            sql.push(where.join(' and '));
        }

        // group
        const group = t._group;
        if (group.length > 0) {
            sql.push('group by');
            sql.push(group.join(','));
        }

        // order
        const order = t._order;
        if (order.length > 0) {
            sql.push('order by');
            sql.push(order.join(','));
        }

        // limit
        const limit = t._limit;
        if (limit) {
            sql.push(limit);
        }

        return sql.join(' ');
    }

    /** 返回一个值，int或null类型 */
    async getInt(paras) {
        const result = await this.getValue(paras);
        if (result !== null) {
            return parseInt(result);
        }
        return null;
    }

    /** 返回一个值 */
    async getValue(paras) {
        const result = await this.getOne(paras);
        if (result !== null) {
            for (const k in result) {
                return result[k];
            }
        }
        return null;
    }

    /** 返回一条数据 */
    async getOne(paras) {
        const result = await this.getList(paras);
        return result.length > 0 ? result[0] : null;
    }

    /** 返回列表 */
    async getList(paras) {
        const t = this;
        const sql = t.toSql();
        return await t.access.query(sql, paras);
    }

    /** 返回翻页列表 */
    async getPage(paras, pageSize, pageIndex) {
        const t = this;

        // rows
        const offset = pageSize * (pageIndex - 1);
        const sql1 = t.toSql() + ` limit ${offset},${pageSize}`;
        const rows = await t.access.query(sql1, paras);

        // total
        t._field = ['count(*) as c'];
        t._order = [];
        const sql2 = t.toSql();
        const total = await t.access.query(sql2, paras);

        return {
            rows: rows,
            total: total[0].c
        }
    }
}

module.exports = Select;
